package org.wso2.usermanager.custom.acegi;

import java.util.Map;

import org.acegisecurity.Authentication;
import org.acegisecurity.providers.AuthenticationProvider;
import org.acegisecurity.providers.UsernamePasswordAuthenticationToken;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.FileSystemXmlApplicationContext;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.readwrite.DefaultRealmConfig;


public class AcegiRealm implements Realm{
	
	private AcegiRealmConfig config = null; 
	private AuthenticationProvider authenticator = null;
	
	public Object getRealmConfiguration() throws UserManagerException{
		DefaultRealmConfig config = null;
		if(config == null){
			config = new DefaultRealmConfig();
		}else{
			config = new DefaultRealmConfig(config);
		}
		return config;
	}

	public void init(Object configBean) throws UserManagerException{
		if(!(configBean instanceof AcegiRealmConfig)){
			return;
		}
		this.config = (AcegiRealmConfig)configBean;
		String fileName = this.config.getAuthenticationProviderBeanMappingFile();
		if(fileName == null){
			throw new UserManagerException("beanMappingNotfound");
		}
		
		ApplicationContext context = new FileSystemXmlApplicationContext(fileName);
		String authProviderId = config.getAuthProviderId();
		if(authProviderId == null){
			throw new UserManagerException("nullAuthProvider");
		}
		authenticator = (AuthenticationProvider)context.getBean(authProviderId);
		
	}
	
	public boolean authenticate(String userName, Object credentials) throws UserManagerException {
		if(!(credentials instanceof String)){
			throw new UserManagerException("unsupportedCredential");
		}
		UsernamePasswordAuthenticationToken token = new UsernamePasswordAuthenticationToken(userName, (String)credentials);
		Authentication result = authenticator.authenticate(token);
		return result.isAuthenticated();
	}
	
	public void addRole(String roleName) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void addUser(String userName, Object credential) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void addUserToRole(String userName, String roleName) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	

	public void authorizeRole(String roleName, String resourceId, String action) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void authorizeUser(String userName, String resourceId, String action) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void clearResourceAuthorizations(String resourceId) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void clearRoleAuthorization(String roleName, String resourceId, String action) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void clearUserAuthorization(String userName, String resourceId, String action) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void copyAuthorizations(String fromResourceId, String toResourceId) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void deleteRole(String roleName) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void deleteUser(String userName) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void denyRole(String roleName, String resourceId, String action) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void denyUser(String userName, String resourceId, String action) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public String[] getAllRoleNames() throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	public String[] getAllUserNames() throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	public String[] getAuthorizedRolesForResource(String resourceId, String action) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	public String[] getAuthorizedUsersForResource(String resourceId, String action) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	public Map getRoleProperties(String roleName) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	public Map getUserProperties(String userName) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	public String[] getUserRoles(String userName) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	public boolean isRoleAuthorized(String roleName, String resourceId, String action) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	public boolean isUserAuthorized(String userName, String resourceId, String action) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	public void removeUserFromRole(String userName, String roleName) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void setRoleProperties(String roleName, Map properties) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void setUserProperties(String userName, Map properties) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}

	public void updateUser(String userName, Object credential) throws UserManagerException {
		throw new UserManagerException("actionNotSupportedByRealm");
		
	}
	

}
