package org.wso2.usermanager.custom.jdbc;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.i18n.Messages;

import java.sql.Connection;
import java.sql.Driver;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.Types;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Properties;

public class JDBCRealm  implements Realm{

	private static Log log = LogFactory.getLog(JDBCRealm.class);
	private static Messages  messages = Messages.getInstance();
	
	protected JDBCRealmConfig config = null;
	protected Driver driver = null;
	protected Connection dbConnection = null;

	public JDBCRealm(){

	}

	public Object getRealmConfiguration() throws UserManagerException{
		JDBCRealmConfig config = null;
		if(config == null){
			config = new JDBCRealmConfig();
		}else{
			config = new JDBCRealmConfig(config);
		}
		return config;
	}

	public void init(Object configBean) throws UserManagerException{
		if(!(configBean instanceof JDBCRealmConfig)){
			return;
		}

		this.config = (JDBCRealmConfig)configBean;
	}

	/**
	 * Open (if necessary) and return a database connection for use by
	 * this Realm.
	 *
	 * @exception SQLException if a database error occurs
	 */
	private Connection open() throws UserManagerException {

		if (dbConnection != null)
			return (dbConnection);

		if(this.config.getConnectionURL() == null){
			throw new UserManagerException("Connection URL is null");
		}


		if (driver == null) {
			try {
				Class clazz = Class.forName(config.getDriverName());
				driver = (Driver) clazz.newInstance();
			} catch (Throwable e) {
				log.debug(e.getMessage(),e);
				e.printStackTrace();
				SQLException exc = new SQLException();
				if(exc.getNextException()!=null){
					Exception nextExecption = exc.getNextException();
					log.debug(messages.getMessage("exceptionOnConnectionOpen", new Object[]{nextExecption.getMessage()}), nextExecption);
				}
				throw new UserManagerException("exceptionOnConnectionOpen", e);
			}
		}

		try {
			Properties props = new Properties();
			if (config.getConnectionUserName() != null)
				props.put("user", config.getConnectionUserName());
			if (config.getConnectionPassword() != null)
				props.put("password", config.getConnectionPassword());
			dbConnection = driver.connect(config.getConnectionURL(), props);
			dbConnection.setAutoCommit(false);
			return (dbConnection);
		} catch (SQLException e) {
			log.debug(e.getMessage(),e);
			throw new UserManagerException(messages.getMessage("exceptionOnConnectionOpen"), e);
		}		

	}
	
	
	public boolean authenticate(String userName, Object credentials) throws UserManagerException {
		if(credentials == null){
			return false;
		}

		if(userName == null){
			return false;
		}

		boolean bValue = false;

		if(!(credentials instanceof String)){
			throw new UserManagerException("Can handle onlyl string type credentials");
		}else{
			try{
				open();
				String password = (String)credentials;
				String stmtValue = this.constructSQLtoReadPassword(userName);
				
				if(log.isDebugEnabled()){
					log.debug(stmtValue);
				}
				
				Statement stmt = dbConnection.createStatement(
						ResultSet.TYPE_SCROLL_INSENSITIVE,
						ResultSet.CONCUR_UPDATABLE);

				ResultSet rs = stmt.executeQuery(stmtValue);

				if(rs.first() == true){
					String value = rs.getString(this.config.getUserCredentialColumn());
					if((value != null) && (value.equals(password))){
						bValue = true;
					}
				}
			} catch (SQLException e) {
				log.error(e.getMessage(),e);
				throw new UserManagerException("exceptionOnAuthenticate");
			}
		}
		return bValue;
	}

	public Map getUserProperties(String userName) throws UserManagerException{
		return readProperties(userName, config.getUserNameColumn(),  config.getUserTable());
	}
	
	public Map getRoleProperties(String roleName)  throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void setUserProperties(String userName, Map properties)  throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public String[] getAllUserNames() throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void setRoleProperties(String roleName, Map properties)  throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");	
	}
	public void addUser(String userName, Object credential) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");	
	}
	public void updateUser(String userName, Object credential) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void deleteUser(String userName) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void addRole(String roleName) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void deleteRole(String roleName) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public String[] getAllRoleNames() throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public String[] getUserRoles(String userName)  throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void addUserToRole(String userName, String roleName) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void removeUserFromRole(String userName, String roleName) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void authorizeUser(String userName, String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void denyUser(String userName, String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public boolean isUserAuthorized(String userName, String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void clearUserAuthorization(String userName, String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void authorizeRole(String roleName, String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void denyRole(String roleName, String resourceId, String action)throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public boolean isRoleAuthorized(String roleName, String action, String resourceId) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void clearRoleAuthorization(String roleName, String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void clearResourceAuthorizations(String resourceId)  throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public String[] getAuthorizedUsersForResource(String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public String[] getAuthorizedRolesForResource(String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	public void copyAuthorizations (String fromResourceId, String toResourceId) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
//	*************************************************************************************
	
	protected String[] selectNames(String entityNameCol, String entityTable) throws UserManagerException{
		//SELECT ColumnName FROM TableName
		String[] names = new String[0];
		StringBuffer sbuff = new StringBuffer();
		sbuff.append("Select ");
		sbuff.append(entityNameCol);
		sbuff.append(" from ");
		sbuff.append(entityTable);
		
		String strStmt = sbuff.toString();
		try {
			open();
			log.debug(strStmt);
			Statement stmt = dbConnection.createStatement(
					ResultSet.TYPE_SCROLL_INSENSITIVE,
					ResultSet.CONCUR_UPDATABLE);

			ResultSet rs = stmt.executeQuery(strStmt);
			if(rs.first() == false){
				return names;
			}

			List lst = new ArrayList();
			while(rs.next()){
				String name = rs.getString(entityNameCol);
				lst.add(name);
			}
			names = (String[])lst.toArray(new String[lst.size()]);
			
		} catch (UserManagerException e) {
			e.printStackTrace();
		} catch (SQLException e) {
			e.printStackTrace();
		}
		
		return names;
	}
	
	protected Map readProperties(String entityName, String entityNameCol, String entityTable) throws UserManagerException{
		Map properties = new HashMap();
		try {
			open();
			String strStmt = constructSQLToReadAttributes(entityName, entityNameCol, entityTable); 
			log.debug(strStmt);
			Statement stmt = dbConnection.createStatement(
					ResultSet.TYPE_SCROLL_INSENSITIVE,
					ResultSet.CONCUR_UPDATABLE);

			ResultSet rs = stmt.executeQuery(strStmt);
			
			Map props = new HashMap();
			if(rs.first() == false){
				return props;
			}
			
			String colNames = config.getColumnNames();
			String[] attrIds = colNames.split(","); 
			ResultSetMetaData mf = rs.getMetaData(); 
			for(int i=0;i<attrIds.length;i++){

				if(attrIds[i].equals(this.config.getUserCredentialColumn())){
					continue;
				}

				int index = i+1; //rs.findColumn(attrIds[i]);
				String colName = mf.getColumnName(index);
				int iType = mf.getColumnType(index);

				switch(iType){
				case Types.INTEGER:
				{
					int value = rs.getInt(index);
					props.put(colName, Integer.toString(value));
				}
				break;
				case Types.DOUBLE:
				{
					double value = rs.getDouble(index);
					props.put(colName, Double.toString(value));
				}
				break;
				case Types.FLOAT:
				{
					float value = rs.getFloat(index);
					props.put(colName, Float.toString(value));
				}
				break;
				case Types.BOOLEAN:
				{
					boolean value = rs.getBoolean(index);
					props.put(colName, Boolean.toString(value));
				}
				case Types.VARCHAR:
				{
					String value = rs.getString(index);
					props.put(colName, value);
				}
				break;
				default:
				{
					//assert false : "Unsupported type";
				}
				}

			}


		} catch (SQLException e) {
			log.error(e.getMessage(),e);
			e.printStackTrace();
		}


		return properties;

	}
	
	private String constructSQLToReadAttributes(String entityName, String entityNameCol, String entityTable) throws UserManagerException{
		StringBuffer sbuff = new StringBuffer("select ");
		String colNames = config.getColumnNames();
		String[] attrIds = colNames.split(",");
		for(int i=0; i<attrIds.length; i++){
			if(attrIds[i].equals(this.config.getUserCredentialColumn())){
				continue;
			}
			sbuff.append(attrIds[i]);
			sbuff.append(", ");
		}
		sbuff.deleteCharAt(sbuff.length()-1);
		sbuff.deleteCharAt(sbuff.length()-1);
		sbuff.append(" from ");
		sbuff.append(entityTable);
		sbuff.append(" where ");
		sbuff.append(entityNameCol);
		sbuff.append(" = '");
		sbuff.append(entityName);
		sbuff.append("'");;

		return sbuff.toString();
	}

	private String constructSQLtoReadPassword(String username){
		StringBuffer sbuff = new StringBuffer("select ");
		sbuff.append(this.config.getUserCredentialColumn());
		sbuff.append(" from ");
		sbuff.append(this.config.getUserTable());
		sbuff.append(" where ");
		sbuff.append(this.config.getUserNameColumn());
		sbuff.append(" = '");
		sbuff.append(username);
		sbuff.append("'");;

		return sbuff.toString();
	}

}

