package org.wso2.usermanager.custom.ldap;

import java.text.MessageFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Hashtable;
import java.util.Map;

import javax.naming.AuthenticationException;
import javax.naming.Context;
import javax.naming.NameNotFoundException;
import javax.naming.NamingEnumeration;
import javax.naming.NamingException;
import javax.naming.directory.Attribute;
import javax.naming.directory.Attributes;
import javax.naming.directory.DirContext;
import javax.naming.directory.InitialDirContext;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.custom.jdbc.JDBCRealmConfig;
import org.wso2.usermanager.i18n.Messages;

public class LDAPRealm implements Realm{
	
	private static Log log = LogFactory.getLog(LDAPRealm.class);
	private static Messages  messages = Messages.getInstance();
	
	protected String[] userPatternArray = null;
	protected MessageFormat[] userPatternFormatArray = null;

	protected DirContext context = null; 
	protected int curUserPattern = 0;

	protected int connectionAttempt = 0;
	protected LDAPRealmConfig config = null;
	protected String driverName = "com.sun.jndi.ldap.LdapCtxFactory";
	
	
	public LDAPRealm(){
		
	}
	
	
	public Object getRealmConfiguration() throws UserManagerException{
		LDAPRealmConfig config = null;
		if(config == null){
			 config = new LDAPRealmConfig();
		}else{
			config = new LDAPRealmConfig(config);
		}
		return config;
	}
	
	public void init(Object configBean) throws UserManagerException{
		if(!(configBean instanceof LDAPRealmConfig)){
			return;
		}
		this.config = (LDAPRealmConfig)configBean;
	}
	
	/**
	 * Authenticates a user against a userstore
	 */
	public boolean authenticate(String userName, Object credentials) throws UserManagerException{
		boolean bValue = false;
		
		if(!(credentials instanceof String)){
			throw new UserManagerException("Can handle onlyl string type credentials");
		}else{
			try {
				open();
				String dn = userPatternFormatArray[curUserPattern].format(new String[] { (String)credentials });
				bValue = this.bindAsUser(this.context, dn, (String)credentials);
			} catch (NamingException e) {
				log.debug(messages.getMessage("exceptionOnAuthenticate"),e);
				throw new UserManagerException("exceptionOnAuthenticate");
			}
		}
		
		return bValue;
	}
	
	
	public Map getUserProperties(String userName) throws UserManagerException{
		log.debug(messages.getMessage("getProperties"));
			
		String[] attrIds = config.getAttributeIds();
				
		Map properties = new HashMap();

		try {
			this.open();

			Attributes results = null;
			// Use pattern or search for user entry
			if (userPatternFormatArray != null) {
				results = getGenericUserByPattern(userName, attrIds);
			}// else {
		//	results = getGenericUserBySearch(userName, attrIds);
		//	}
						
			if(results != null){
				NamingEnumeration names = results.getAll();
				while(names.hasMoreElements()){
					Attribute attr = (Attribute)names.nextElement();
					properties.put(attr.getID(), attr.get().toString());
				}
			}
			
		} catch (NamingException e) {
			log.debug(messages.getMessage("getProperties"), e);
			throw new UserManagerException("unexpected",e);
		}
		
		return properties;

	}
	
	
	public void setUserProperties(String userName, Map properties)  throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	
	public void setRoleProperties(String roleName, Map properties)  throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	
	public Map getRoleProperties(String roleName)  throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
		
	
	/**
	 * Add a user to the userstore.
	 */
	public void addUser(String userName, Object credential) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	
	/**
	 * Updates a user in the userstore.
	 * Updates his password and properties. 
	 */
	public void updateUser(String userName, Object credential) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	
	/**
	 * Deletes a user in the userstore
	 */
	public void deleteUser(String userName) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Returns an array of Usernames from the userstore
	 */
	public String[] getAllUserNames() throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Add a role to the userstore
	 */
	public void addRole(String roleName) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Delete role from userstore given the rolename.
	 */
	public void deleteRole(String roleName) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Returns an array of all rolenames int the userstore
	 */
	public String[] getAllRoleNames() throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Returns an array of Role given a user name
	 */
	public String[] getUserRoles(String userName)  throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Adds a User to the Role
	 */
	public void addUserToRole(String userName, String roleName) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Removes a user from role
	 */
	public void removeUserFromRole(String userName, String roleName) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Allows the user to perform the given action on the given resource
	 */
	public void authorizeUser(String userName, String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Blocks the user performing the given action on the given resource
	 */
	public void denyUser(String userName, String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Checks whether the user is allowed to perform the given action on the given resource
	 */
	public boolean isUserAuthorized(String userName, String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Removes the "user, action, resource" relationship. 
	 * User, action and resource will exist, but the relationship will be removed.
	 */
	public void clearUserAuthorization(String userName, String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Allows the role to perform the given action on the given resource
	 */
	public void authorizeRole(String roleName, String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Blocks the role performing the given action on the given resource
	 */
	public void denyRole(String roleName, String resourceId, String action)throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Checks whether the role is allowed to perform the given action on the given resource
	 */
	public boolean isRoleAuthorized(String roleName, String action, String resourceId) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Removes the "role, action, resource" relationship. 
	 * Role, action and resource will exist, but the relationship will be removed.
	 */
	public void clearRoleAuthorization(String roleName, String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Removes all authorizations specified on a resource.
	 */
	public void clearResourceAuthorizations(String resourceId)  throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Returns a list of users authorized to perform an action on a resource
	 */
	public String[] getAuthorizedUsersForResource(String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Returns a list of roles authorized to perform an action on a resource
	 */
	public String[] getAuthorizedRolesForResource(String resourceId, String action) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}

	/**
	 * Copies all authorization of "fromResourceId" to "toResourceId".
	 * All means all - including user authorizations and role authorizations.
	 */
	public void copyAuthorizations (String fromResourceId, String toResourceId) throws UserManagerException{
		throw new UserManagerException("actionNotSupportedByRealm");
	}
	
	
	/**
	 * Open (if necessary) and return a connection to the configured
	 * directory server for this Realm.
	 *
	 * @exception NamingException if a directory server error occurs
	 */
	protected DirContext open() throws NamingException {

		if (context != null)
			return (context);

		try {
			// Ensure that we have a directory context available
			context = new InitialDirContext(getDirectoryContextEnvironment());
		} catch (Exception e) {
			connectionAttempt = 1;
			log.debug(e.getMessage());
			e.printStackTrace();
			context = new InitialDirContext(getDirectoryContextEnvironment());
		} finally {
			// reset it in case the connection times out.
			connectionAttempt = 0;
		}

		return (context);

	}
	
	

	


	/**
	 * Use the <code>UserPattern</code> configuration attribute to
	 * locate the directory entry for the user with the specified
	 * username and return a User object; otherwise return
	 * <code>null</code>.
	 *
	 * @param context The directory context
	 * @param username The username
	 * @param attrIds String[]containing names of attributes to
	 * retrieve.
	 *
	 * @exception NamingException if a directory server error occurs
	 */
	protected Attributes getGenericUserByPattern(String userName,
			String[] attrIds)
	throws NamingException, UserManagerException {

		log.info("getGenericUserByPattern");


		if (userName == null || userPatternFormatArray[curUserPattern] == null)
			return (null);

		// Form the dn from the user pattern
		String dn = userPatternFormatArray[curUserPattern].format(new String[] { userName });

		// Get required attributes from user entry
		Attributes attrs = null;
	
			open();
			attrs = context.getAttributes(dn, attrIds);


		return attrs;

	}
	
	
	protected Hashtable getDirectoryContextEnvironment() {

		Hashtable env = new Hashtable();
		
		env.put(Context.INITIAL_CONTEXT_FACTORY, driverName);
		
		if (config.getConnectionName() != null)
			env.put(Context.SECURITY_PRINCIPAL, config.getConnectionName());
		
		if (config.getConnectionPass() != null)
			env.put(Context.SECURITY_CREDENTIALS, config.getConnectionPass());
		
		if (config.getConnectionUrl() != null && connectionAttempt == 0)
			env.put(Context.PROVIDER_URL, config.getConnectionUrl());
		
		return env;

	}

	
    /**
     * Check credentials by binding to the directory as the user
     *
     * @param context The directory context
     * @param user The User to be authenticated
     * @param credentials Authentication credentials
     *
     * @exception NamingException if a directory server error occurs
     */
     protected boolean bindAsUser(DirContext context,
                                  String dn,
                                  String credentials)
         throws NamingException {
         Attributes attr;

         if (credentials == null ||dn == null)
             return (false);

         // Validate the credentials specified by the user
         if (log.isTraceEnabled()) {
             log.trace("  validating credentials by binding as the user");
        }
        open();
        
        // Set up security environment to bind as the user
        context.addToEnvironment(Context.SECURITY_PRINCIPAL, dn);
        context.addToEnvironment(Context.SECURITY_CREDENTIALS, credentials);

        // Elicit an LDAP bind operation
        boolean validated = false;
        try {
            if (log.isTraceEnabled()) {
                log.trace("  binding as "  + dn);
            }
            attr = context.getAttributes("", null);
            validated = true;
        }
        catch (AuthenticationException e) {
            if (log.isDebugEnabled()) {
                log.debug("  bind attempt failed", e);
            }
        }

        // Restore the original security environment
        
        if (config.getConnectionName() != null) {
            context.addToEnvironment(Context.SECURITY_PRINCIPAL, config.getConnectionName());
        } else {
            context.removeFromEnvironment(Context.SECURITY_PRINCIPAL);
        }

        if (config.getConnectionPass() != null) {
            context.addToEnvironment(Context.SECURITY_CREDENTIALS, config.getConnectionPass());
        }
        else {
            context.removeFromEnvironment(Context.SECURITY_CREDENTIALS);
        }

        return validated;
     }
     
//********************************** starting private methods *******************************************
	
	
	
	private void setUserPattern(String userPattern) {
		if(userPattern == null)
			this.userPatternArray = null;

		userPatternArray = parseUserPatternString(userPattern);
		int len = this.userPatternArray.length;
		userPatternFormatArray = new MessageFormat[len];
		for (int i=0; i < len; i++) {
			userPatternFormatArray[i] =
				new MessageFormat(userPatternArray[i]);
		}
	}
	
	
	/**
	 * Given a string containing LDAP patterns for user locations (separated by
	 * parentheses in a pseudo-LDAP search string format -
	 * "(location1)(location2)", returns an array of those paths.  Real LDAP
	 * search strings are supported as well (though only the "|" "OR" type).
	 *
	 * @param userPatternString - a string LDAP search paths surrounded by
	 * parentheses
	 */
	protected String[] parseUserPatternString(String userPatternString) {

		if (userPatternString != null) {
			ArrayList pathList = new ArrayList();
			int startParenLoc = userPatternString.indexOf('(');
			if (startParenLoc == -1) {
				// no parens here; return whole thing
				return new String[] {userPatternString};
			}
			int startingPoint = 0;
			while (startParenLoc > -1) {
				int endParenLoc = 0;
				// weed out escaped open parens and parens enclosing the
				// whole statement (in the case of valid LDAP search
				// strings: (|(something)(somethingelse))
				while ( (userPatternString.charAt(startParenLoc + 1) == '|') ||
						(startParenLoc != 0 && userPatternString.charAt(startParenLoc - 1) == '\\') ) {
					startParenLoc = userPatternString.indexOf("(", startParenLoc+1);
				}
				endParenLoc = userPatternString.indexOf(")", startParenLoc+1);
				// weed out escaped end-parens
				while (userPatternString.charAt(endParenLoc - 1) == '\\') {
					endParenLoc = userPatternString.indexOf(")", endParenLoc+1);
				}
				String nextPathPart = userPatternString.substring
				(startParenLoc+1, endParenLoc);
				pathList.add(nextPathPart);
				startingPoint = endParenLoc+1;
				startParenLoc = userPatternString.indexOf('(', startingPoint);
			}
			return (String[])pathList.toArray(new String[] {});
		}
		return null;

	}
	

}
