package org.wso2.usermanager.readwrite;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;
import java.util.Properties;
import java.util.Map.Entry;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;

public class DefaultRealm implements Realm{


	protected DefaultRealmConfig config = null;
	protected Driver driver = null;
	protected Connection dbConnection = null;
	protected Properties sqlBundle = null;
	protected long idRole = 0;
	protected long idUser = 0;
	protected long idRoleAttribute = 0;
	protected long idUserAttribute =0;
	protected long idPermission = 0;
	protected long idUserPermission = 0;
	protected long idRolePermission = 0;


	private PreparedStatement addUserStmt = null;
	private PreparedStatement updateUserStmt = null;
	private PreparedStatement deleteUserStmt  = null;
	private PreparedStatement getUserPropertiesStmt  = null;
	private PreparedStatement setUserPropertiesStmt  = null;
	private PreparedStatement deleteUserPropsStmt = null;
	
	private PreparedStatement addRoleStmt  = null;
	private PreparedStatement deleteRoleStmt  = null;
	private PreparedStatement getRolePropertiesStmt  = null;
	private PreparedStatement setRolePropertiesStmt  = null;
	private PreparedStatement deleteRolePropsStmt = null;
	

	private PreparedStatement getAllRoleNamesStmt  = null;
	private PreparedStatement getAllUserNamesStmt  = null;

	private PreparedStatement getUserRolesStmt  = null;
	private PreparedStatement addUserToRoleStmt  = null;
	private PreparedStatement removeUserFromRoleStmt  = null;

	private PreparedStatement getUserStmt  = null;

	private PreparedStatement addUserPermissionStmt  = null;
	private PreparedStatement addRolePermissionStmt  = null;

	private PreparedStatement isRoleAuthorizedStmt  = null;
	private PreparedStatement isUserAuthorizedStmt  = null;

	private PreparedStatement clearUserAuthorizationStmt  = null;
	private PreparedStatement clearRoleAuthorizationStmt  = null;
	private PreparedStatement clearResourceAuthorizationsStmt  = null;
	private PreparedStatement getAuthorizedUsersForResourceStmt  = null;
	private PreparedStatement getAuthorizedRolesForResourceStmt  = null;
	private PreparedStatement getUserIdStmt= null;
	private PreparedStatement getRolIdStmt= null;
	
	private PreparedStatement addPermissionStmt = null;
	private PreparedStatement getPermissionStmt = null;
	
	public DefaultRealm(){


	}

	private static Log log = LogFactory.getLog(DefaultRealm.class);

	public Object getRealmConfiguration() throws UserManagerException{
		DefaultRealmConfig config = null;
		if(config == null){
			config = new DefaultRealmConfig();
		}else{
			config = new DefaultRealmConfig(config);
		}
		return config;
	}

	public void init(Object configBean) throws UserManagerException{
		if(!(configBean instanceof DefaultRealmConfig)){
			return;
		}
		this.config = (DefaultRealmConfig)configBean;
		String fileName = this.config.getSqlFile();
		if(fileName == null){
			throw new RuntimeException("Config not found");
		}
		try {
			File file = new File(fileName);
			FileInputStream in = new FileInputStream(file);
			this.sqlBundle = new Properties();
			sqlBundle.load(in);
			open();
		} catch (IOException e) {
			throw new RuntimeException("sqlFileNotFound");
		}

	}

	public void addUser(String userName, Object credential) throws UserManagerException{
		if(!(credential instanceof String)) {
			throw new UserManagerException("Credential type not support");
		}

		try {
			addUserStmt.setString(1, userName);
			addUserStmt.setString(2, (String)credential);
			idUser = idUser+1;
			addUserStmt.setLong(3, idUser);
			addUserStmt.executeUpdate();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}

	}

	public void updateUser(String userName, Object credential) throws UserManagerException{
		try {
			updateUserStmt.setString(1, (String)credential);
			updateUserStmt.setString(2, userName);
			updateUserStmt.executeUpdate();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}
	}

	public void deleteUser(String userName) throws UserManagerException{

		try {
			deleteUserStmt.setString(1,userName);
			deleteUserStmt.executeUpdate();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}

	}

	public Map getUserProperties(String userName) throws UserManagerException{
		Map props = new HashMap();
		try {
			getUserPropertiesStmt.setString(1, userName);
			ResultSet rs = getUserPropertiesStmt.executeQuery();

			while(rs.next()){
				String name = rs.getString(COLUMN_ATTR_NAME);
				String value = rs.getString(COLUMN_ATTR_VALUE);
				props.put(name, value);
			}
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorReadingFromUserStore",e);
		}

		return props;

	}

	public void setUserProperties(String userName, Map properties)  throws UserManagerException{

		long userid = this.getUserId(userName);
		if(userid == -1){
			throw new UserManagerException("nullUser");
		}
			
		try {	
			deleteUserPropsStmt.setLong(1, userid);
			deleteUserPropsStmt.executeUpdate();

			Iterator ite = properties.entrySet().iterator();
			while(ite.hasNext()){
				Entry entry = (Entry)ite.next();
				String key = (String)entry.getKey();
				String value = (String)entry.getValue();
				if(value != null){
					idUserAttribute= idUserAttribute+1;
					setUserPropertiesStmt.setString(1, key);
					setUserPropertiesStmt.setString(2, value);
					setUserPropertiesStmt.setLong(3, userid);
					setUserPropertiesStmt.setLong(4,idUserAttribute);
					setUserPropertiesStmt.executeUpdate();
				}
			}
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}

	}

	public String[] getAllUserNames() throws UserManagerException{
		String[] names = new String[1];
		try {
			ResultSet rs = getAllUserNamesStmt.executeQuery();
			List lst = new LinkedList();
			while(rs.next()){
				lst.add(rs.getString(COLUMN_USER_NAME));
			}
			if(lst.size()>0){
				names = (String[])lst.toArray(new String[lst.size()]);
			}
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorReadingFromUserStore",e);
		}
		return names;
	}

	public void addRole(String roleName) throws UserManagerException{
		try {
			idRole = idRole +1;
			addRoleStmt.setString(1, roleName);
			addRoleStmt.setLong(2, idRole);
			addRoleStmt.execute();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}
	}

	public void deleteRole(String roleName) throws UserManagerException{
		try {
			deleteRoleStmt.setString(1, roleName);
			deleteRoleStmt.execute();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}
	}

	public Map getRoleProperties(String roleName)  throws UserManagerException{
		
		Map props = new HashMap();
		try {
			getRolePropertiesStmt.setString(1, roleName);
			ResultSet rs = getRolePropertiesStmt.executeQuery();

			while(rs.next()){
				String name = rs.getString(COLUMN_ATTR_NAME);
				String value = rs.getString(COLUMN_ATTR_VALUE);
				props.put(name, value);
			}
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorReadingFromUserStore",e);
		}

		return props;

	}

	public void setRoleProperties(String roleName, Map properties)  throws UserManagerException{
		long roleid = this.getRoleId(roleName);
		if(roleid == -1){
			throw new UserManagerException("nullRole");
		}
			
		try {	
			deleteRolePropsStmt.setLong(1, roleid);
			deleteRolePropsStmt.executeUpdate();

			Iterator ite = properties.entrySet().iterator();
			while(ite.hasNext()){
				Entry entry = (Entry)ite.next();
				String key = (String)entry.getKey();
				String value = (String)entry.getValue();
				if(value != null){
					idRoleAttribute= idRoleAttribute+1;
					setRolePropertiesStmt.setString(1, key);
					setRolePropertiesStmt.setString(2, value);
					setRolePropertiesStmt.setLong(3, roleid);
					setRolePropertiesStmt.setLong(4,idRoleAttribute);
					setRolePropertiesStmt.executeUpdate();
				}
			}
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}
	}

	public String[] getAllRoleNames() throws UserManagerException{
		String[] names = new String[0];
		try {
			ResultSet rs = getAllRoleNamesStmt.executeQuery();
			List lst = new LinkedList();
			while(rs.next()){
				lst.add(rs.getString(COLUMN_ROLE_NAME));
			}
			if(lst.size()>0){
				names = (String[])lst.toArray(new String[lst.size()]);
			}
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorReadingFromUserStore",e);
		}
		return names;
	}

	public String[] getUserRoles(String userName)  throws UserManagerException{
		String[] names = new String[0];
		try {
			getUserRolesStmt.setString(1, userName);
			ResultSet rs = getUserRolesStmt.executeQuery();
			List lst = new LinkedList();
			while(rs.next()){
				lst.add(rs.getString(COLUMN_ROLE_NAME));
			}
			if(lst.size()>0){
				names = (String[])lst.toArray(new String[lst.size()]);
			}
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorReadingFromUserStore",e);
		}
		return names;

	}

	public void addUserToRole(String userName, String roleName) throws UserManagerException{
		long userid = this.getUserId(userName);
		long roleid = this.getRoleId(roleName);

		if(roleid==-1 || userid == -1){
			throw new UserManagerException("nullData");
		}

		try {
			addUserToRoleStmt.setLong(1, userid);
			addUserToRoleStmt.setLong(2, roleid);
			addUserToRoleStmt.executeUpdate();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}
	}

	public void removeUserFromRole(String userName, String roleName) throws UserManagerException{
		try {
			removeUserFromRoleStmt.setString(1, userName);
			removeUserFromRoleStmt.setString(2, roleName);
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}
	}

	public boolean authenticate(String userName, Object credentials) throws UserManagerException{
		boolean isAuth = false;

		try {
			getUserStmt.setString(1, userName);
			ResultSet rs = getUserStmt.executeQuery();
			if(rs.next()){
				String dbCred = rs.getString(COLUMN_CREDENTIAL);
				if(credentials != null && credentials.equals(dbCred)){
					isAuth = true;
				}
			}
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorReadingFromUserStore",e);
		}
		return isAuth;
	}

	public void authorizeUser(String userName, String resourceId, String action) throws UserManagerException{

		long userId = this.getUserId(userName); 

		if(userId == -1){
			throw new UserManagerException("nullUser");
		}

		long permissionId = this.getOrAddPermissionId(resourceId, action);

		idUserPermission=idUserPermission+1;
		short allow = 1;
		try {
			addUserPermissionStmt.setLong(1,permissionId);
			addUserPermissionStmt.setShort(2, allow);
			addUserPermissionStmt.setLong(3, userId);
			addUserPermissionStmt.setLong(4, idUserPermission);
			addUserPermissionStmt.executeUpdate();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}

	}

	public void denyUser(String userName, String resourceId, String action) throws UserManagerException{
		long permissionId = this.getOrAddPermissionId(resourceId, action);

		long userId = this.getUserId(userName);

		if(userId == -1){
			throw new UserManagerException("nullUser");
		}

		idUserPermission=idUserPermission+1;
		short allow = 0;
		try {
			addUserPermissionStmt.setLong(1,permissionId);
			addUserPermissionStmt.setShort(2, allow);
			addUserPermissionStmt.setLong(3, userId);
			addUserPermissionStmt.setLong(4, idUserPermission);
			addUserPermissionStmt.executeUpdate();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}
	}

	public boolean isUserAuthorized(String userName, String resourceId, String action) throws UserManagerException{
		boolean isAuthorized = false;
		try {
			isUserAuthorizedStmt.setString(1, resourceId);
			isUserAuthorizedStmt.setString(2, action);
			isUserAuthorizedStmt.setString(3, userName);
			ResultSet rs = isUserAuthorizedStmt.executeQuery();
			if(rs.next()){
				isAuthorized = rs.getBoolean(COLUMN_IS_ALLOWED);
			}
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorReadingFromUserStore",e);
		}
		return isAuthorized;
	}

	public void clearUserAuthorization(String userName, String resourceId, String action) throws UserManagerException{
		try {
			clearUserAuthorizationStmt.setString(1, resourceId);
			clearUserAuthorizationStmt.setString(2, action);
			clearUserAuthorizationStmt.setString(3, userName);
			clearUserAuthorizationStmt.executeUpdate();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}
	}

	public void authorizeRole(String roleName, String resourceId, String action) throws UserManagerException{
		long roleId = this.getRoleId(roleName); 

		if(roleId == -1){
			throw new UserManagerException("nullRole");
		}

		long permissionId = this.getOrAddPermissionId(resourceId, action);

		idRolePermission = idRolePermission+1;
		short allow = 1;
		try {
			addRolePermissionStmt.setLong(1,permissionId);
			addRolePermissionStmt.setShort(2, allow);
			addRolePermissionStmt.setLong(3, roleId);
			addRolePermissionStmt.setLong(4, idRolePermission);
			addRolePermissionStmt.executeUpdate();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}

	}

	public void denyRole(String roleName, String resourceId, String action)throws UserManagerException{
		long roleId = this.getRoleId(roleName); 

		if(roleId == -1){
			throw new UserManagerException("nullRole");
		}

		long permissionId = this.getOrAddPermissionId(resourceId, action);

		idRolePermission = idRolePermission+1;
		short allow = 0;
		try {
			addRolePermissionStmt.setLong(1,permissionId);
			addRolePermissionStmt.setShort(2, allow);
			addRolePermissionStmt.setLong(3, roleId);
			addRolePermissionStmt.setLong(4, idRolePermission);
			addRolePermissionStmt.executeUpdate();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}
	}

	public boolean isRoleAuthorized(String roleName, String resourceId, String action) throws UserManagerException{
		boolean isAuthorized = false;
		try {
			isRoleAuthorizedStmt.setString(1, resourceId);
			isRoleAuthorizedStmt.setString(2, action);
			isRoleAuthorizedStmt.setString(3, roleName);
			ResultSet rs = isRoleAuthorizedStmt.executeQuery();
			if(rs.next()){
				isAuthorized = rs.getBoolean(COLUMN_IS_ALLOWED);
			}
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorReadingFromUserStore",e);
		}
		return isAuthorized;
	}

	public void clearRoleAuthorization(String roleName, String resourceId, String action) throws UserManagerException{

		try {
			clearRoleAuthorizationStmt.setString(1, resourceId);
			clearRoleAuthorizationStmt.setString(2, action);
			clearRoleAuthorizationStmt.setString(3, roleName);
			clearRoleAuthorizationStmt.executeUpdate();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}

	}

	public void clearResourceAuthorizations(String resourceId)  throws UserManagerException{
		try {
			clearResourceAuthorizationsStmt.setString(1, resourceId);
			clearResourceAuthorizationsStmt.executeUpdate();
			dbConnection.commit();
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}
	}

	public String[] getAuthorizedUsersForResource(String resourceId, String action) throws UserManagerException{
		String[] names = new String[0]; 
		try{
			getAuthorizedUsersForResourceStmt.setString(1, resourceId);
			getAuthorizedUsersForResourceStmt.setString(2, action);
			ResultSet rs = getAuthorizedUsersForResourceStmt.executeQuery();
			List lst = new LinkedList();
			while(rs.next()){
				lst.add(rs.getString(COLUMN_USER_NAME));
			}
			if(lst.size()>0){
				names = (String[])lst.toArray(new String[lst.size()]);
			}
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorReadingFromUserStore",e);
		}
		return names;
	}

	public String[] getAuthorizedRolesForResource(String resourceId, String action) throws UserManagerException{
		String[] names = new String[0]; 
		try{
			getAuthorizedRolesForResourceStmt.setString(1, resourceId);
			getAuthorizedRolesForResourceStmt.setString(2, action);
			ResultSet rs = getAuthorizedRolesForResourceStmt.executeQuery();
			List lst = new LinkedList();
			while(rs.next()){
				lst.add(rs.getString(COLUMN_ROLE_NAME));
			}
			if(lst.size()>0){
				names = (String[])lst.toArray(new String[lst.size()]);
			}
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorReadingFromUserStore",e);
		}
		return names;
	}

	public void copyAuthorizations (String fromResourceId, String toResourceId) throws UserManagerException{
		//todo
	}



	/**
	 * Returns the long id of the entity
	 * @param strStmt
	 * @return
	 */
	protected long getUserId(String userName) throws UserManagerException{
		long id = -1;
		try {
			getUserIdStmt.setString(1, userName);
			ResultSet rs = getUserIdStmt.executeQuery();
			if(rs.next()){
				id = rs.getLong(COLUMN_ID);
			}
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorReadingFromUserStore",e); 
		}
		return id;
	}

	protected long getRoleId(String roleName) throws UserManagerException{
		long id = -1;
		try {
			getRolIdStmt.setString(1, roleName);
			ResultSet rs = getRolIdStmt.executeQuery();
			if(rs.next()){
				id = rs.getLong(COLUMN_ID);
			}
		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorReadingFromUserStore",e);
		}
		return id;
	}

	/**
	 * Gets the permission if it exists, else create the permission and returns the id.
	 * @return
	 */
	protected long getOrAddPermissionId(String resourceId, String action) throws UserManagerException{
		long permissionId = -1;
		try {
			getPermissionStmt.setString(1, resourceId);
			getPermissionStmt.setString(2, action);
			
			ResultSet rs = getPermissionStmt.executeQuery();

			if(rs.next()){
				permissionId = rs.getInt(COLUMN_ID);
			}

			if(permissionId <1 ){
				//no permission so add it
				idPermission = this.idPermission+1;
				addPermissionStmt.setString(1, resourceId);
				addPermissionStmt.setString(2, action);
				addPermissionStmt.setLong(3, idPermission);
				addPermissionStmt.executeUpdate();
				dbConnection.commit();
				permissionId = idPermission;
			}

		} catch (SQLException e) {
			log.debug(e);
			throw new UserManagerException("errorModifyingUserStore",e);
		}

		return permissionId;

	}

	/**
	 * Open (if necessary) and return a database connection for use by
	 * this Realm.
	 *
	 * @exception SQLException if a database error occurs
	 */
	private void open() throws UserManagerException{

		if (dbConnection != null)
			return;

		if(this.config.getConnectionURL() == null){
			throw new UserManagerException("Connection URL is null");
		}


		if (driver == null) {
			try {
				Class clazz = Class.forName(config.getDriverName());
				driver = (Driver) clazz.newInstance();
			} catch (Throwable e) {
				log.debug(e.getMessage(),e);
				e.printStackTrace();
				SQLException exc = new SQLException();
				if(exc.getNextException()!=null){
					Exception nextExecption = exc.getNextException();
					log.debug("The so called next exception is" + nextExecption.getMessage(), nextExecption);
				}
				throw new UserManagerException("exceptionOnConnectionOpen", e);
			}
		}



		try {
			Properties props = new Properties();
			if (config.getConnectionUserName() != null)
				props.put("user", config.getConnectionUserName());
			if (config.getConnectionPassword() != null)
				props.put("password", config.getConnectionPassword());
			dbConnection = driver.connect(config.getConnectionURL(), props);
			dbConnection.setAutoCommit(false);
			loadMaxIds();
			loadPreparedStatements();
		} catch (SQLException e) {
			log.debug(e.getMessage(),e);
			throw new UserManagerException("exceptionOnConnectionOpen", e);
		}

		return;


	}

	protected String getSQL(String sqlName){
		String strStmt = (String)sqlBundle.get(sqlName);
		return strStmt;
	}


	private void loadMaxIds() throws SQLException{
		String strStmt = this.getSQL(MAX_USER);
		Statement stmt = dbConnection.createStatement(ResultSet.TYPE_SCROLL_INSENSITIVE, ResultSet.CONCUR_UPDATABLE);
		ResultSet rs = stmt.executeQuery(strStmt);
		if(rs.next()){
			idUser = rs.getInt(1);
		}

		strStmt = this.getSQL(MAX_ROLE);
		rs = stmt.executeQuery(strStmt);
		if(rs.next()){
			idRole = rs.getInt(1);
		}
		strStmt = this.getSQL(MAX_USER_ATTRIBUTE);
		rs = stmt.executeQuery(strStmt);
		if(rs.next()){
			idUserAttribute = rs.getInt(1);
		}
		strStmt = this.getSQL(MAX_ROLE_ATTRIBUTE);
		rs = stmt.executeQuery(strStmt);
		if(rs.next()){
			idRoleAttribute = rs.getInt(1);
		}
		strStmt = this.getSQL(MAX_PERMISSION);
		rs = stmt.executeQuery(strStmt);
		if(rs.next()){
			idPermission = rs.getInt(1);
		}
		strStmt = this.getSQL(MAX_ROLE_PERMISSION);
		rs = stmt.executeQuery(strStmt);
		if(rs.next()){
			idRolePermission = rs.getInt(1);
		}
		strStmt = this.getSQL(MAX_USER_PERMISSION);
		rs = stmt.executeQuery(strStmt);
		if(rs.next()){
			idUserPermission = rs.getInt(1);
		}
		stmt.close();
	}

	private void loadPreparedStatements() throws SQLException{
		addUserStmt = dbConnection.prepareStatement(this.getSQL(ADD_USER));
		updateUserStmt = dbConnection.prepareStatement(this.getSQL(UPDATE_USER));
		deleteUserStmt  = dbConnection.prepareStatement(this.getSQL(DELETE_USER));
		getUserPropertiesStmt  = dbConnection.prepareStatement(this.getSQL(GET_USER_ATTRIBUTES));
		setUserPropertiesStmt  = dbConnection.prepareStatement(this.getSQL(ADD_USER_ATTRIBUTE));

		addRoleStmt  = dbConnection.prepareStatement(this.getSQL(ADD_ROLE));
		deleteRoleStmt  = dbConnection.prepareStatement(this.getSQL(DELETE_ROLE));
		getRolePropertiesStmt  = dbConnection.prepareStatement(this.getSQL(GET_ROLE_ATTRIBUTES));
		setRolePropertiesStmt  = dbConnection.prepareStatement(this.getSQL(ADD_ROLE_ATTRIBUTE));
		
		addPermissionStmt = dbConnection.prepareStatement(this.getSQL(ADD_PERMISSION));
		
		getAllRoleNamesStmt  = dbConnection.prepareStatement(this.getSQL(GET_ROLES_ALL));
		getAllUserNamesStmt  = dbConnection.prepareStatement(this.getSQL(GET_USERS_ALL));

		getUserRolesStmt  = dbConnection.prepareStatement(this.getSQL(GET_USER_ROLES));
		addUserToRoleStmt  = dbConnection.prepareStatement(this.getSQL(ADD_USER_ROLE));
		removeUserFromRoleStmt  = dbConnection.prepareStatement(this.getSQL(DELETE_USER_ROLE));

		getUserStmt = dbConnection.prepareStatement(this.getSQL(GET_USER));
		addUserPermissionStmt  = dbConnection.prepareStatement(this.getSQL(ADD_USER_PERMISSION));
		addRolePermissionStmt  = dbConnection.prepareStatement(this.getSQL(ADD_ROLE_PERMISSION));

		isUserAuthorizedStmt  = dbConnection.prepareStatement(this.getSQL(GET_USER_ALLOWED));
		isRoleAuthorizedStmt  = dbConnection.prepareStatement(this.getSQL(GET_ROLE_ALLOWED));

		clearUserAuthorizationStmt  = dbConnection.prepareStatement(this.getSQL(DELETE_USER_PERMISSION));
		clearRoleAuthorizationStmt  = dbConnection.prepareStatement(this.getSQL(DELETE_ROLE_PERMISSION));
		clearResourceAuthorizationsStmt  = dbConnection.prepareStatement(this.getSQL(DELETE_PERMISSION_RESOURCE));
		getAuthorizedUsersForResourceStmt  = dbConnection.prepareStatement(this.getSQL(GET_RESOURCE_PERMISSION_USERS));
		getAuthorizedRolesForResourceStmt  = dbConnection.prepareStatement(this.getSQL(GET_RESOURCE_PERMISSION_ROLES));

		getUserIdStmt=dbConnection.prepareStatement(this.getSQL(GET_USER_ID));
		getRolIdStmt=dbConnection.prepareStatement(this.getSQL(GET_ROLE_ID));
		
		deleteUserPropsStmt = dbConnection.prepareStatement(this.getSQL(DELETE_USER_ATTRIBUTE));
		deleteRolePropsStmt = dbConnection.prepareStatement(this.getSQL(DELETE_ROLE_ATTRIBUTE));
		
		getPermissionStmt = dbConnection.prepareStatement(this.getSQL(GET_PERMISSION));
		
	}

	private String encloseInQuotes(String value){
		StringBuilder sb = new StringBuilder("'");
		sb.append(value);
		sb.append("'");
		return sb.toString();
	}


	public static final String ADD_USER = "add.user";
	public static final String ADD_ROLE = "add.role";
	public static final String ADD_USER_ROLE = "add.user.role";
	public static final String ADD_USER_ATTRIBUTE = "add.user.attribute";
	public static final String ADD_ROLE_ATTRIBUTE = "add.role.attribute";
	public static final String ADD_PERMISSION = "add.permission";
	public static final String ADD_ROLE_PERMISSION = "add.role.permission";
	public static final String ADD_USER_PERMISSION = "add.user.permission";


	public static final String 	UPDATE_USER = "update.user";

	public static final String 	DELETE_USER = "delete.user";
	public static final String 	DELETE_ROLE = "delete.role";
	public static final String 	DELETE_USER_ROLE = "delete.user.role";
	public static final String 	DELETE_ROLE_ATTRIBUTE = "delete.role.attribute";
	public static final String 	DELETE_USER_ATTRIBUTE = "delete.user.attribute";
	public static final String 	DELETE_PERMISSION_RESOURCE = "delete.permission.resource";
	public static final String 	DELETE_USER_PERMISSION = "delete.user.permission";
	public static final String 	DELETE_ROLE_PERMISSION = "delete.role.permission";

	public static final String 	GET_USER = "get.user";
	public static final String 	GET_ROLE = "get.role";
	public static final String 	GET_USER_ROLES = "get.user.roles";
	public static final String 	GET_ROLE_ATTRIBUTES = "get.role.attributes";
	public static final String 	GET_USER_ATTRIBUTES = "get.user.attributes";
	public static final String 	GET_PERMISSION = "get.permission";
	public static final String 	GET_ROLE_ALLOWED = "get.role.allowed";
	public static final String 	GET_USER_ALLOWED = "get.user.allowed";
	public static final String 	GET_RESOURCE_PERMISSION_ROLES = "get.resource.role";
	public static final String 	GET_RESOURCE_PERMISSION_USERS = "get.resource.user";
	public static final String 	GET_ROLE_PERMISSION = "get.role.permission";
	public static final String 	GET_USER_PERMISSION = "get.user.permission";
	public static final String 	GET_ROLES_ALL = "get.roles.all";
	public static final String 	GET_USERS_ALL = "get.users.all";
	public static final String 	GET_USER_ID = "get.user.id";
	public static final String 	GET_ROLE_ID = "get.role.id";


	public static final String COLUMN_ATTR_NAME =  "attr_name";
	public static final String COLUMN_ATTR_VALUE = "attr_value";

	public static final String COLUMN_ID = "id";
	public static final String COLUMN_USER_NAME = "user_name";
	public static final String COLUMN_ROLE_NAME = "role_name";
	public static final String COLUMN_IS_ALLOWED = "is_allowed";
	public static final String COLUMN_CREDENTIAL = "password";

	public static final String MAX_USER = "max.user";
	public static final String MAX_ROLE = "max.role";
	public static final String MAX_USER_ATTRIBUTE = "max.user.attribute";
	public static final String MAX_ROLE_ATTRIBUTE = "max.role.attribute";
	public static final String MAX_PERMISSION = "max.permission";
	public static final String MAX_ROLE_PERMISSION = "max.role.permission";
	public static final String MAX_USER_PERMISSION = "max.user.permission";

}
