package org.wso2.usermanager.verification.email;

import java.sql.Connection;
import java.sql.DatabaseMetaData;
import java.sql.Driver;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Properties;
import java.util.Random;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.AddressException;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.wso2.usermanager.Realm;
import org.wso2.usermanager.UserManagerException;
import org.wso2.usermanager.readwrite.DefaultRealm;

import com.sun.org.apache.xerces.internal.impl.dv.util.Base64;

public class EmailVerifier {

	private static Log log = LogFactory.getLog(DefaultRealm.class);

	private static Realm realm = null;
	private static EmailVerifierConfig config = null;

	public static final String DRIVER = "org.apache.derby.jdbc.EmbeddedDriver";
	public static final String PROTOCOL = "jdbc:derby:";

	private static PreparedStatement addUserStmt = null;
	private static PreparedStatement getUserByConfStmt = null;
	private static Connection dbConnection = null;


	/**
	 * Configures the Email Verifier
	 * @param realmInstance - The realm instance where the email verifier is going to call addUser
	 * @param verifierConfig - Email Verifierification config
	 */
	public static void init(Realm realmInstance, EmailVerifierConfig verifierConfig) throws UserManagerException{

		if((realmInstance==null) && (verifierConfig == null)){
			throw new RuntimeException("Invalid or null data provided");
		}

		realm = realmInstance;
		config = verifierConfig;
		createDatabase();
		compileStmts();

	}

	public boolean confirmUser(String value) throws UserManagerException{
		if(realm == null){
			throw new UserManagerException("initVerifier");
		}
		
		boolean isConfirmed = false;
		try {
			getUserByConfStmt.setString(1, value);
			ResultSet rs = getUserByConfStmt.executeQuery();
			if(rs.next()){
				String username = rs.getString("email_address");
				String password = rs.getString("password");
				realm.addUser(username, password);
				isConfirmed = true;
			}
		} catch (SQLException e) {
			throw new UserManagerException("error",e);
		}
		
		return isConfirmed;
	}

	public void requestUserVerification(String emailAddress, String password) throws UserManagerException{
		if(config == null){
			throw new UserManagerException("initVerifier");
		}

		try {
			Random random = new Random();
			byte[] temp = new byte[16];
			random.nextBytes(temp);
			String confString = Base64.encode(temp);
			confString = confString.trim();
				
			String epr = config.getRegistrationServiceEPR();
			sendEmail(confString, emailAddress, epr);
			addUserStmt.setString(1, emailAddress);
			addUserStmt.setString(2, password);
			addUserStmt.setString(3, confString);
			addUserStmt.executeUpdate();
			dbConnection.commit();
		} catch (SQLException e) {
			throw new UserManagerException("pendingAdd",e);
		}

	}


	protected void sendEmail(String confString, String emailAddr, String epr) throws UserManagerException{
		Properties props = new Properties();
		props.put(EmailVerifierConfig.HOST, config.getHost());
		props.put(EmailVerifierConfig.PORT, Integer.toString(config.getPort()));
		Session session = Session.getDefaultInstance(props, null);

		try {
			// Construct the message
			Message msg = new MimeMessage(session);
			msg.setFrom(new InternetAddress(config.getFromAddress()));
			msg.setRecipient(Message.RecipientType.TO, new InternetAddress(emailAddr));
			msg.setSubject("EmailVerification");

			String responseMessage = getResponseMessage(confString, epr);
			msg.setText(responseMessage);

			log.debug("Sending confirmation mail to " + emailAddr);
			log.debug("Verification url : " + responseMessage);
			// Send the message
			Transport.send(msg);
			log.debug("Sending confirmation mail to " + emailAddr + "DONE");
		} catch (AddressException e) {
			throw new UserManagerException("sendingMailProblems", e);
		} catch (MessagingException e) {
			throw new UserManagerException("sendingMailProblems", e);
		}
	}


	protected static void createDatabase() throws UserManagerException{
		try {
			Class clazz = Class.forName(DRIVER);
			Driver driver = (Driver) clazz.newInstance();

			Properties props = new Properties();

			String connectionURL = PROTOCOL + "UnVerifiedUserDatabase;create=true";
			dbConnection = driver.connect(connectionURL, props);


			Statement stmt = dbConnection.createStatement(
					ResultSet.TYPE_SCROLL_INSENSITIVE,
					ResultSet.CONCUR_UPDATABLE);

			DatabaseMetaData dbmd = dbConnection.getMetaData();
			ResultSet rs = dbmd.getTables(null, null, "PENDING_USERS", null);

			if(rs.next() == false){
				log.debug("Creating a new table in the database.");
				stmt = dbConnection.createStatement(
						ResultSet.TYPE_SCROLL_INSENSITIVE,
						ResultSet.CONCUR_UPDATABLE);
				stmt.executeUpdate("create table PENDING_USERS(email_address varchar(255) not null, password varchar(255) not null, confirmation varchar(255) not null unique, primary key (email_address))");
				stmt.executeUpdate("insert into PENDING_USERS(email_address, password, confirmation) values('admin', 'dummyPass', 'adminConfirmation')");
				stmt.executeUpdate("insert into PENDING_USERS(email_address, password, confirmation) values('root', 'dummyPass', 'rootConfirmation')");
				dbConnection.commit();
			}else{
				log.debug("Database and table already found.");

			}
			dbConnection.commit();
		} catch (Exception e) {
			throw new UserManagerException("verifierUserDatabaseInit",e);
		}
	}

	protected static void compileStmts() throws UserManagerException{
		try {
			addUserStmt = dbConnection.prepareStatement("insert into PENDING_USERS(email_address, password, confirmation) values(?, ?, ?)");
			getUserByConfStmt = dbConnection.prepareStatement("select * from PENDING_USERS where confirmation=?");
		} catch (SQLException e) {
			throw new UserManagerException("error",e);
		}
	}

	private String getResponseMessage(String confString, String epr) {
		return "Please point your browser to : " + epr +"?confirmation=" +confString;
	}




}
